//
//  ViewController.m
//  地图与定位
//
//  Created by Iracle Zhang on 7/28/16.
//  Copyright © 2016 Iracle. All rights reserved.
//

#import "ViewController.h"
@import CoreLocation;
@import MapKit;

@interface ViewController () <CLLocationManagerDelegate, MKMapViewDelegate>

//定位管理
@property (nonatomic, strong) CLLocationManager *locationManager;
//地图
@property (nonatomic, strong) MKMapView *mapView;

@end

@implementation ViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    [self setupUI];
    [self loadLocationService];
}
//加载UI
- (void)setupUI {
    //重定位
    self.navigationItem.leftBarButtonItem= [[UIBarButtonItem alloc] initWithTitle:@"定位" style:UIBarButtonItemStylePlain target:self action:@selector(barButtonTaped:)];
    self.navigationItem.leftBarButtonItem.tag = 100;
    
    //位置详细信息
    self.navigationItem.rightBarButtonItem= [[UIBarButtonItem alloc] initWithTitle:@"详情" style:UIBarButtonItemStylePlain target:self action:@selector(barButtonTaped:)];
    self.navigationItem.rightBarButtonItem.tag = 101;
    
    //地图类型切换
    UISegmentedControl *mapTypeSegmentControl = [[UISegmentedControl alloc] initWithItems:@[@"普通", @"卫星", @"混合"]];
    mapTypeSegmentControl.bounds = CGRectMake(0, 0, 200, 25);
    mapTypeSegmentControl.selectedSegmentIndex = 0;
    [mapTypeSegmentControl addTarget:self action:@selector(progressSegmentControl:) forControlEvents:UIControlEventValueChanged];
    
    UIBarButtonItem *segmentItem = [[UIBarButtonItem alloc] initWithCustomView:mapTypeSegmentControl];
    
    UIBarButtonItem *flexibleItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemFlexibleSpace target:nil action:NULL];
    self.toolbarItems = @[flexibleItem, segmentItem, flexibleItem];
    
    //实例化地图
    _mapView = [[MKMapView alloc] initWithFrame:self.view.bounds];
    //是否显示用户标注
    _mapView.showsUserLocation = YES;
    //地图类型
    _mapView.mapType = MKMapTypeStandard;
    //设置代理
    _mapView.delegate = self;
    [self.view addSubview:_mapView];
    
    //长按手势
    UILongPressGestureRecognizer *longPress = [[UILongPressGestureRecognizer alloc] initWithTarget:self action:@selector(longPressMapView:)];
    longPress.minimumPressDuration = 1;
    [self.view addGestureRecognizer:longPress];
    
    
}
//定位服务
- (void)loadLocationService {
    //先配置info.plist 关键字 NSLocationWhenInUseUsageDescription NSLocationAlwaysUsageDescription 配置设置选项
    /**
     NSLocationAlwaysUsageDescription 后台一直开启定位服务
     NSLocationWhenInUseUsageDescription 只在程序运行时开启定位服务
     */
    
    //实例化位置管理器
    _locationManager = [[CLLocationManager alloc] init];
    //判断异常
    if (![CLLocationManager locationServicesEnabled]) {
         NSLog(@"定位服务不可用，可能是GPS模块坏了，请你尽快换手机");
    }
    //弹出权限警告框
    [_locationManager requestWhenInUseAuthorization];
    _locationManager.delegate = self;
    //定位精度
    _locationManager.desiredAccuracy = kCLLocationAccuracyBest;
    //位置更新最小距离
    _locationManager.distanceFilter = kCLDistanceFilterNone;
    //启动定位
    [_locationManager startUpdatingLocation];
    
}

#pragma mark -- Event
- (void)barButtonTaped:(UIBarButtonItem *)sender {
    NSInteger index = sender.tag - 100;
    if (index == 0) {
        //定位
        [_locationManager startUpdatingLocation];
    } else {
        //详情
        
    }
}

- (void)progressSegmentControl:(UISegmentedControl *)sender {
    
    _mapView.mapType = sender.selectedSegmentIndex;
}

#pragma mark -- CLLocationManagerDelegate

- (void)locationManager:(CLLocationManager *)manager
     didUpdateLocations:(NSArray<CLLocation *> *)locations {
    
     NSLog(@"位置信息：%@", manager.location);
    //停止定位
    [manager stopUpdatingLocation];
    [self updateMapViewWithLocaiton:manager.location];
    
}

//更新地图
- (void)updateMapViewWithLocaiton:(CLLocation *)location {
    
    //跨度
    MKCoordinateSpan span = MKCoordinateSpanMake(0.05, 0.05);
    //范围
    MKCoordinateRegion region= MKCoordinateRegionMake(location.coordinate, span);
    //配置地图显示区域
    [_mapView setRegion:region animated:YES];
    
    //配置当前位置的信息
    _mapView.userLocation.title = @"当前位置";
    _mapView.userLocation.subtitle = [NSString stringWithFormat:@"%.2f, %.2f", location.coordinate.latitude, location.coordinate.longitude];
    
}

- (void)longPressMapView:(UILongPressGestureRecognizer *)gesture {
    if (gesture.state == UIGestureRecognizerStateBegan) {
        //获取手势坐标
        CGPoint point = [gesture locationInView:gesture.view];
        //视图坐标转换为经纬度坐标
        CLLocationCoordinate2D coordinate = [_mapView convertPoint:point toCoordinateFromView:gesture.view];
        //利用经纬度获取 CLLocation
        CLLocation *location = [[CLLocation alloc] initWithLatitude:coordinate.latitude longitude:coordinate.longitude];
        
        //地理位置逆编码
        [self veverseGeoLocation:location completion:^(BOOL sucess, id content) {
            if (sucess) {
                 NSLog(@"%@",content);
                //大头针
                MKPointAnnotation *annotation = [[MKPointAnnotation alloc] init];
                annotation.coordinate = coordinate;
                //标题
                annotation.title = content[@"Name"];
                [_mapView addAnnotation:annotation];
            }
        }];
        
    }
}

//地理位置逆编码：把经纬度信息编码成格式化的地理位置信息

- (void)veverseGeoLocation:(CLLocation *)location completion:(void(^)(BOOL sucess, id content))completion {
    
    CLGeocoder *coder = [[CLGeocoder alloc] init];
    //逆编码方法，后台线程执行
    [coder reverseGeocodeLocation:location completionHandler:^(NSArray<CLPlacemark *> * _Nullable placemarks, NSError * _Nullable error) {
        //completion block 需要的两个参数
        BOOL sucess;
        id content = nil;
        
        if (error) {
            sucess = NO;
            content = error.localizedDescription;
            
        } else {
            sucess = YES;
            CLPlacemark *placeMark = placemarks.firstObject;
            content = placeMark.addressDictionary;
        }
        //执行 block
        completion(sucess, content);
        
    }];
}

#pragma mark -- MKMapViewDelegate
- (MKAnnotationView *)mapView:(MKMapView *)mapView viewForAnnotation:(id<MKAnnotation>)annotation {
    
    if ([annotation isKindOfClass:[MKUserLocation class]]) {
        return  nil;
    }
    
    //定义标识符
    static NSString *viewIdentifer = @"AnnotationView";
    MKPinAnnotationView *annotationView = (MKPinAnnotationView *)[mapView dequeueReusableAnnotationViewWithIdentifier:viewIdentifer];
    if (!annotationView) {
        annotationView = [[MKPinAnnotationView alloc] initWithAnnotation:annotation reuseIdentifier:viewIdentifer];
        //颜色
        annotationView.pinTintColor = [UIColor purpleColor];
        //是否动画
        annotationView.animatesDrop = YES;
        //是否需要点击
        annotationView.canShowCallout = YES;
        //配置详情按钮
        UIButton *detailButton = [UIButton buttonWithType:UIButtonTypeInfoDark];
        annotationView.rightCalloutAccessoryView  = detailButton;
        
    }
    
    return annotationView;
}

//点击弹出框
- (void)mapView:(MKMapView *)mapView annotationView:(MKAnnotationView *)view calloutAccessoryControlTapped:(UIControl *)control {
    
     NSLog(@"当前位置被点击了");
}



@end
















